# currency_selector.py
#
# Copyright 2023 Ideve Core
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later



import re
from gi.repository import Adw, Gdk, GObject, Gtk
from ..currency_selector_row.currency_selector_row import CurrencySelectorRow

@Gtk.Template(resource_path='/io/github/idevecore/Valuta/components/currency_selector/index.ui')
class CurrencySelector(Adw.Bin):
    __gtype_name__ = 'CurrencySelector'
    __gsignals__ = {
        'user-selection-changed': (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
    }

    selected = GObject.Property(type=str)
    insight = Gtk.Template.Child()
    button = Gtk.Template.Child()
    label = Gtk.Template.Child()
    popover = Gtk.Template.Child()
    search = Gtk.Template.Child()
    scroll = Gtk.Template.Child()
    currency_list = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.model = None
        self.search.set_key_capture_widget(self.popover)
        key_events = Gtk.EventControllerKey.new()
        key_events.connect('key-pressed', self.on_key_pressed)
        self.search.add_controller(key_events)

    def bind_models(self, currencies):
        self.model = currencies
        self.filter = Gtk.CustomFilter()
        self.filter.set_filter_func(self.filter_currencies)
        sorter = Gtk.CustomSorter.new(self.sort_currencies)
        sorted_model = Gtk.SortListModel.new(model=self.model, sorter=sorter)
        filter_model = Gtk.FilterListModel.new(sorted_model, self.filter)
        self.currency_list.bind_model(filter_model, self.create_currency_row)

    def set_insight(self, code):
        if self.selected == 'auto':
            self.insight.props.label = f'({self._get_currency_name(code)})'

    def _get_currency_name(self, code):
        return self.model.names_func(code)

    @Gtk.Template.Callback()
    def _on_selected_changed(self, _self, _pspec):
        if self.model is not None:
            self.model.set_selected(self.selected)
            self.label.props.label = self.selected
            self.insight.props.label = ''

    def set_selected(self, code):
        if self.model is not None:
            self.selected = code
            self.model.set_selected(self.selected)
            self.label.props.label = self.selected
            self.insight.props.label = ''


    @Gtk.Template.Callback()
    def _activated(self, _list, row):
        self.popover.popdown()
        self.selected = row.currency.code
        self.emit('user-selection-changed')

    @Gtk.Template.Callback()
    def _popover_show(self, _popover):
        self.search.grab_focus()

    @Gtk.Template.Callback()
    def _popover_closed(self, _popover):
        vscroll = self.scroll.get_vadjustment()
        vscroll.props.value = 0
        self.search.props.text = ''
  
    def filter_currencies(self, item):
        search = self.search.get_text()
        return bool(re.search(search, f'{item} - {item.name}', re.IGNORECASE))

    def sort_currencies(self, currency_a, currency_b, _data):
        a = currency_a.name.lower()
        b = currency_b.name.lower()
        return (a > b) - (a < b)

    def create_currency_row(self, currency):
        return CurrencySelectorRow(currency)

    @Gtk.Template.Callback()
    def _on_search(self, _entry):
        self.filter.emit('changed', Gtk.FilterChange.DIFFERENT)

    @Gtk.Template.Callback()
    def _on_search_activate(self, _entry):
        if self.search.props.text:
            row = self.currency_list.get_row_at_index(0)
            if row:
                self.currency_list.emit('row-activated', row)
        return Gdk.EVENT_PROPAGATE

    def on_key_pressed(self,_controller, keyval, _keycode, _mod):
        if keyval == Gdk.KEY_Escape:
            self.popover.popdown()
        elif keyval == Gdk.KEY_Down:
            return Gdk.EVENT_STOP




#!/usr/bin/python3
# define.in
#
# Copyright 2023 Ideve Core
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import gettext

APP_ID = "io.github.idevecore.Valuta"
VERSION = '1.4.2'
PROFILE = ""
IS_DEVEL = PROFILE == "Devel"
RES_PATH = '/io/github/idevecore/Valuta'
SUFFIX = "(Devel)" if IS_DEVEL else ""
BASE_URL_LANG_PREFIX = '&hl=en&lr=lang_en'

gettext.install('valuta', '/usr/share/locale')
_ = gettext.gettext

#The symbols is in Latin mode (based on dataset from wikipedia

CODES = {
  "AFN": {
    "name": _("Afghan Afghani"),
    "flag": "🇦🇫",
    "providers": "1,2",
    "symbol": "AFN",
    "about": "https://en.wikipedia.org/wiki/Afghan_afghani",
  },
  "ALL": {
    "name": _("Albanian Lek"),
    "flag": "🇦🇱",
    "providers": "1,2",
    "symbol": "L",
    "about": "https://en.wikipedia.org/wiki/Albanian_lek",
  },
  "DZD": {
    "name": _("Algerian Dinar"),
    "flag": "🇩🇿",
    "providers": "1,2",
    "symbol": "DZD",
    "about": "https://en.wikipedia.org/wiki/Algerian_dinar",
  },
  "AOA": {
    "name": _("Angolan Kwanza"),
    "flag": "🇦🇴",
    "providers": "1,2",
    "symbol": "Kz",
    "about": "https://en.wikipedia.org/wiki/Angolan_kwanza",
  },
  "ARS": {
    "name": _("Argentine Peso"),
    "flag": "🇦🇷",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Argentine_peso",
  },
  "AMD": {
    "name": _("Armenian Dram"),
    "flag": "🇦🇲",
    "providers": "1,2",
    "symbol": "֏",
    "about": "https://en.wikipedia.org/wiki/Armenian_dram",
  },
  "AWG": {
    "name": _("Aruban Florin"),
    "flag": "🇦🇼",
    "providers": "1,2",
    "symbol": "ƒ",
    "about": "https://en.wikipedia.org/wiki/Aruban_florin",
  },
  "AUD": {
    "name": _("Australian Dollar"),
    "flag": "🇦🇺",
    "providers": "0,1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Australian_dollar",
  },
  "AZN": {
    "name": _("Azerbaijani Manat"),
    "flag": "🇦🇿",
    "providers": "1,2",
    "symbol": "₼",
    "about": "https://en.wikipedia.org/wiki/Azerbaijani_manat",
  },
  "BSD": {
    "name": _("Bahamian Dollar"),
    "flag": "🇧🇸",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Bahamian_dollar",
  },
  "BHD": {
    "name": _("Bahraini Dinar"),
    "flag": "🇧🇭",
    "providers": "1,2",
    "symbol": "BHD",
    "about": "https://en.wikipedia.org/wiki/Bahraini_dinar",
  },
  "BBD": {
    "name": _("Bajan dollar"),
    "flag": "🇧🇧",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Barbadian_dollar",
  },
  "BDT": {
    "name": _("Bangladeshi Taka"),
    "flag": "🇧🇩",
    "providers": "1,2",
    "symbol": "৳",
    "about": "https://en.wikipedia.org/wiki/Bangladeshi_taka",
  },
  "BYR": {
    "name": _("Belarusian Ruble"),
    "flag": "🇧🇾",
    "providers": "1",
    "symbol": "Rbl",
    "about": "https://en.wikipedia.org/wiki/Belarusian_ruble",
  },
  "BYN": {
    "name": _("Belarusian Ruble"),
    "flag": "🇧🇾",
    "providers": "1,2",
    "symbol": "Rbl",
    "about": "https://en.wikipedia.org/wiki/Belarusian_ruble",
  },
  "BZD": {
    "name": _("Belize Dollar"),
    "flag": "🇧🇿",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Belize_dollar",
  },
  "BMD": {
    "name": _("Bermudan Dollar"),
    "flag": "🇧🇲",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Bermudian_dollar",
  },
  "BTN": {
    "name": _("Bhutan currency"),
    "flag": "🇧🇹",
    "providers": "1,2",
    "symbol": "Nu.",
    "about": "https://en.wikipedia.org/wiki/Bhutanese_ngultrum",
  },
  "BTC": {
    "name": _("Bitcoin"),
    "flag": "₿",
    "providers": "1,2",
    "symbol": "₿",
    "about": "https://en.wikipedia.org/wiki/Bitcoin",
  },
  "BCH": {
    "name": _("Bitcoin Cash"),
    "flag": "₿",
    "providers": "1",
    "symbol": "₿",
    "about": "https://en.wikipedia.org/wiki/Bitcoin_Cash",
  },
  "BOB": {
    "name": _("Bolivian Boliviano"),
    "flag": "🇧🇴",
    "providers": "1,2",
    "symbol": "Bs",
    "about": "https://en.wikipedia.org/wiki/Bolivian_boliviano",
  },
  "BAM": {
    "name": _("Bosnia-Herzegovina Convertible Mark"),
    "flag": "🇧🇦",
    "providers": "1,2",
    "symbol": "KM",
    "about": "https://en.wikipedia.org/wiki/Bosnia_and_Herzegovina_convertible_mark",
  },
  "BWP": {
    "name": _("Botswanan Pula"),
    "flag": "🇧🇼",
    "providers": "1,2",
    "symbol": "P",
    "about": "https://en.wikipedia.org/wiki/Botswana_pula",
  },
  "BRL": {
    "name": _("Brazilian Real"),
    "flag": "🇧🇷",
    "providers": "0,1,2",
    "symbol": "R$",
    "about": "https://en.wikipedia.org/wiki/Brazilian_real",
  },
  "BND": {
    "name": _("Brunei Dollar"),
    "flag": "🇧🇳",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Brunei_dollar",
  },
  "BGN": {
    "name": _("Bulgarian Lev"),
    "flag": "🇧🇬",
    "providers": "0,1,2",
    "symbol": "лв",
    "about": "https://en.wikipedia.org/wiki/Bulgarian_lev",
  },
  "BIF": {
    "name": _("Burundian Franc"),
    "flag": "🇧🇮",
    "providers": "1,2",
    "symbol": "FBu",
    "about": "https://en.wikipedia.org/wiki/Burundian_franc",
  },
  "XPF": {
    "name": _("CFP Franc"),
    "flag": "F",
    "providers": "1,2",
    "symbol": "F",
    "about": "https://en.wikipedia.org/wiki/CFP_franc",
  },
  "KHR": {
    "name": _("Cambodian riel"),
    "flag": "🇰🇭",
    "providers": "1,2",
    "symbol": "៛",
    "about": "https://en.wikipedia.org/wiki/Cambodian_riel",
  },
  "CAD": {
    "name": _("Canadian Dollar"),
    "flag": "🇨🇦",
    "providers": "0,1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Canadian_dollar",
  },
  "CVE": {
    "name": _("Cape Verdean Escudo"),
    "flag": "🇨🇻",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Cape_Verdean_escudo",
  },
  "KYD": {
    "name": _("Cayman Islands Dollar"),
    "flag": "🇰🇾",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Cayman_Islands_dollar",
  },
  "XAF": {
    "name": _("Central African CFA franc"),
    "flag": "F.CFA",
    "providers": "1,2",
    "symbol": "F.CFA",
    "about": "https://en.wikipedia.org/wiki/Central_African_CFA_franc",
  },
  "CLP": {
    "name": _("Chilean Peso"),
    "flag": "🇨🇱",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Chilean_peso",
  },
  "CLF": {
    "name": _("Chilean Unit of Account (UF)"),
    "flag": "🇨🇱",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Unidad_de_Fomento",
  },
  "CNY": {
    "name": _("Chinese Yuan"),
    "flag": "🇨🇳",
    "providers": "0,1,2",
    "symbol": "¥",
    "about": "https://en.wikipedia.org/wiki/Renminbi",
  },
  "CNH": {
    "name": _("Chinese Yuan (offshore)"),
    "flag": "🇨🇳",
    "providers": "1,2",
    "symbol": "¥",
    "about": "https://en.wikipedia.org/wiki/Renminbi",
  },
  "COP": {
    "name": _("Colombian Peso"),
    "flag": "🇨🇴",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Colombian_peso",
  },
  "KMF": {
    "name": _("Comorian franc"),
    "flag": "🇰🇲",
    "providers": "1,2",
    "symbol": "FC",
    "about": "https://en.wikipedia.org/wiki/Comorian_franc",
  },
  "CDF": {
    "name": _("Congolese Franc"),
    "flag": "🇨🇩",
    "providers": "1,2",
    "symbol": "FC",
    "about": "https://en.wikipedia.org/wiki/Congolese_franc",
  },
  "CRC": {
    "name": _("Costa Rican Colón"),
    "flag": "🇨🇷",
    "providers": "1,2",
    "symbol": "₡",
    "about": "https://en.wikipedia.org/wiki/Costa_Rican_col%C3%B3n",
  },
  "HRK": {
    "name": _("Croatian Kuna"),
    "flag": "🇭🇷",
    "providers": "1,2",
    "symbol": "kn",
    "about": "https://en.wikipedia.org/wiki/Croatian_kuna",
  },
  "CUP": {
    "name": _("Cuban Peso"),
    "flag": "🇨🇺",
    "providers": "1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/Cuban_peso",
  },
  "CZK": {
    "name": _("Czech Koruna"),
    "flag": "🇨🇿",
    "providers": "0,1,2",
    "symbol": "Kč",
    "about": "https://en.wikipedia.org/wiki/Czech_koruna",
  },
  "DKK": {
    "name": _("Danish Krone"),
    "flag": "🇩🇰",
    "providers": "0,1,2",
    "symbol": "kr",
    "about": "https://en.wikipedia.org/wiki/Danish_krone",
  },
  "DJF": {
    "name": _("Djiboutian Franc"),
    "flag": "🇩🇯",
    "providers": "1,2",
    "symbol": "Fdj",
    "about": "https://en.wikipedia.org/wiki/Djiboutian_franc",
  },
  "DOP": {
    "name": _("Dominican Peso"),
    "flag": "🇩🇴",
    "providers": "1,2",
    "symbol": "RD$",
    "about": "https://en.wikipedia.org/wiki/Dominican_peso",
  },
  "XCD": {
    "name": _("East Caribbean Dollar"),
    "flag": "EC$",
    "providers": "1,2",
    "symbol": "EC$",
    "about": "https://en.wikipedia.org/wiki/Eastern_Caribbean_dollar",
  },
  "EGP": {
    "name": _("Egyptian Pound"),
    "flag": "🇪🇬",
    "providers": "1,2",
    "symbol": "EGP",
    "about": "https://en.wikipedia.org/wiki/Egyptian_pound",
  },
  "ETH": {
    "name": _("Ether"),
    "flag": "ETH",
    "providers": "1,2",
    "symbol": "ETH",
    "about": "https://en.wikipedia.org/wiki/Ether",
  },
  "ETB": {
    "name": _("Ethiopian Birr"),
    "flag": "🇪🇹",
    "providers": "1,2",
    "symbol": "ETB",
    "about": "https://en.wikipedia.org/wiki/Ethiopian_birr",
  },
  "EUR": {
    "name": _("Euro"),
    "flag": "🇪🇺",
    "providers": "0,1,2",
    "symbol": "€",
    "about": "https://en.wikipedia.org/wiki/Euro",
  },
  "FJD": {
    "name": _("Fijian Dollar"),
    "flag": "🇫🇯",
    "providers": "1,2",
    "symbol": "FJD",
    "about": "https://en.wikipedia.org/wiki/Fijian_dollar",
  },
  "GMD": {
    "name": _("Gambian dalasi"),
    "flag": "🇬🇲",
    "providers": "1,2",
    "symbol": "GMD",
    "about": "https://en.wikipedia.org/wiki/Gambian_dalasi",
  },
  "GEL": {
    "name": _("Georgian Lari"),
    "flag": "🇬🇪",
    "providers": "1,2",
    "symbol": "GEL",
    "about": "https://en.wikipedia.org/wiki/Georgian_lari",
  },
  "GHC": {
    "name": _("Ghanaian Cedi"),
    "flag": "GHC",
    "providers": "1",
    "symbol": "GHC",
    "about": "https://en.wikipedia.org/wiki/Ghanaian_cedi",
  },
  "GHS": {
    "name": _("Ghanaian Cedi"),
    "flag": "🇬🇭",
    "providers": "1,2",
    "symbol": "GHS",
    "about": "https://en.wikipedia.org/wiki/Ghanaian_cedi",
  },
  "GIP": {
    "name": _("Gibraltar Pound"),
    "flag": "🇬🇮",
    "providers": "1,2",
    "symbol": "GIP",
    "about": "https://en.wikipedia.org/wiki/Gibraltar_pound",
  },
  "GTQ": {
    "name": _("Guatemalan Quetzal"),
    "flag": "🇬🇹",
    "providers": "1,2",
    "symbol": "GTQ",
    "about": "https://en.wikipedia.org/wiki/Guatemalan_quetzal",
  },
  "GNF": {
    "name": _("Guinean Franc"),
    "flag": "🇬🇳",
    "providers": "1,2",
    "symbol": "GNF",
    "about": "https://en.wikipedia.org/wiki/Guinean_franc",
  },
  "GYD": {
    "name": _("Guyanaese Dollar"),
    "flag": "🇬🇾",
    "providers": "1,2",
    "symbol": "GYD",
    "about": "https://en.wikipedia.org/wiki/Guyanese_dollar",
  },
  "HTG": {
    "name": _("Haitian Gourde"),
    "flag": "🇭🇹",
    "providers": "1,2",
    "symbol": "HTG",
    "about": "https://en.wikipedia.org/wiki/Haitian_gourde",
  },
  "HNL": {
    "name": _("Honduran Lempira"),
    "flag": "🇭🇳",
    "providers": "1,2",
    "symbol": "HNL",
    "about": "https://en.wikipedia.org/wiki/Honduran_lempira",
  },
  "HKD": {
    "name": _("Hong Kong Dollar"),
    "flag": "🇭🇰",
    "providers": "0,1,2",
    "symbol": "HKD",
    "about": "https://en.wikipedia.org/wiki/Hong_Kong_dollar",
  },
  "HUF": {
    "name": _("Hungarian Forint"),
    "flag": "🇭🇺",
    "providers": "0,1,2",
    "symbol": "HUF",
    "about": "https://en.wikipedia.org/wiki/Hungarian_forint",
  },
  "ISK": {
    "name": _("Icelandic Króna"),
    "flag": "🇮🇸",
    "providers": "0,1,2",
    "symbol": "ISK",
    "about": "https://en.wikipedia.org/wiki/Icelandic_kr%C3%B3na",
  },
  "INR": {
    "name": _("Indian Rupee"),
    "flag": "🇮🇳",
    "providers": "0,1,2",
    "symbol": "₹",
    "about": "https://en.wikipedia.org/wiki/Indian_rupee",
  },
  "IDR": {
    "name": _("Indonesian Rupiah"),
    "flag": "🇮🇩",
    "providers": "0,1,2",
    "symbol": "IDR",
    "about": "https://en.wikipedia.org/wiki/Indian_rupee",
  },
  "IRR": {
    "name": _("Iranian Rial"),
    "flag": "🇮🇷",
    "providers": "1,2",
    "symbol": "IRR",
    "about": "https://en.wikipedia.org/wiki/Iranian_rial",
  },
  "IQD": {
    "name": _("Iraqi Dinar"),
    "flag": "🇮🇶",
    "providers": "1,2",
    "symbol": "IQD",
    "about": "https://en.wikipedia.org/wiki/Iraqi_dinar",
  },
  "ILS": {
    "name": _("Israeli New Shekel"),
    "flag": "🇮🇱",
    "providers": "0,1,2",
    "symbol": "₪",
    "about": "https://en.wikipedia.org/wiki/Israeli_new_shekel",
  },
  "JMD": {
    "name": _("Jamaican Dollar"),
    "flag": "🇯🇲",
    "providers": "1,2",
    "symbol": "JMD",
    "about": "https://en.wikipedia.org/wiki/Jamaican_dollar",
  },
  "JPY": {
    "name": _("Japanese Yen"),
    "flag": "🇯🇵",
    "providers": "0,1,2",
    "symbol": "¥",
    "about": "https://en.wikipedia.org/wiki/Japanese_yen",
  },
  "JOD": {
    "name": _("Jordanian Dinar"),
    "flag": "🇯🇴",
    "providers": "1,2",
    "symbol": "JOD",
    "about": "https://en.wikipedia.org/wiki/Jordanian_dinar",
  },
  "KZT": {
    "name": _("Kazakhstani Tenge"),
    "flag": "🇰🇿",
    "providers": "1,2",
    "symbol": "KZT",
    "about": "https://en.wikipedia.org/wiki/Kazakhstani_tenge",
  },
  "KES": {
    "name": _("Kenyan Shilling"),
    "flag": "🇰🇪",
    "providers": "1,2",
    "symbol": "KES",
    "about": "https://en.wikipedia.org/wiki/Kenyan_shilling",
  },
  "KWD": {
    "name": _("Kuwaiti Dinar"),
    "flag": "🇰🇼",
    "providers": "1,2",
    "symbol": "KWD",
    "about": "https://en.wikipedia.org/wiki/Kuwaiti_dinar",
  },
  "KGS": {
    "name": _("Kyrgyzstani Som"),
    "flag": "🇰🇬",
    "providers": "1,2",
    "symbol": "KGS",
    "about": "https://en.wikipedia.org/wiki/Kyrgyz_som",
  },
  "LAK": {
    "name": _("Laotian Kip"),
    "flag": "🇱🇦",
    "providers": "1,2",
    "symbol": "LAK",
    "about": "https://en.wikipedia.org/wiki/Lao_kip",
  },
  "LBP": {
    "name": _("Lebanese pound"),
    "flag": "🇱🇧",
    "providers": "1,2",
    "symbol": "LBP",
    "about": "https://en.wikipedia.org/wiki/Lebanese_pound",
  },
  "LSL": {
    "name": _("Lesotho loti"),
    "flag": "🇱🇸",
    "providers": "1,2",
    "symbol": "LSL",
    "about": "https://en.wikipedia.org/wiki/Lesotho_loti",
  },
  "LRD": {
    "name": _("Liberian Dollar"),
    "flag": "🇱🇷",
    "providers": "1,2",
    "symbol": "LRD",
    "about": "https://en.wikipedia.org/wiki/Liberian_dollar",
  },
  "LYD": {
    "name": _("Libyan Dinar"),
    "flag": "🇱🇾",
    "providers": "1,2",
    "symbol": "LYD",
    "about": "https://en.wikipedia.org/wiki/Libyan_dinar",
  },
  "LTC": {
    "name": _("Litecoin"),
    "flag": "LTC",
    "providers": "1,2",
    "symbol": "LTC",
    "about": "https://en.wikipedia.org/wiki/Litecoin",
  },
  "MOP": {
    "name": _("Macanese Pataca"),
    "flag": "🇲🇴",
    "providers": "1,2",
    "symbol": "MOP",
    "about": "https://en.wikipedia.org/wiki/Macanese_pataca",
  },
  "MKD": {
    "name": _("Macedonian Denar"),
    "flag": "🇲🇰",
    "providers": "1,2",
    "symbol": "MKD",
    "about": "https://en.wikipedia.org/wiki/Macedonian_denar",
  },
  "MGA": {
    "name": _("Malagasy Ariary"),
    "flag": "🇲🇬",
    "providers": "1,2",
    "symbol": "MGA",
    "about": "https://en.wikipedia.org/wiki/Malagasy_ariary",
  },
  "MWK": {
    "name": _("Malawian Kwacha"),
    "flag": "🇲🇼",
    "providers": "1,2",
    "symbol": "MWK",
    "about": "https://en.wikipedia.org/wiki/Malawian_kwacha",
  },
  "MYR": {
    "name": _("Malaysian Ringgit"),
    "flag": "🇲🇾",
    "providers": "0,1,2",
    "symbol": "MYR",
    "about": "https://en.wikipedia.org/wiki/Malaysian_ringgit",
  },
  "MVR": {
    "name": _("Maldivian Rufiyaa"),
    "flag": "🇲🇻",
    "providers": "1,2",
    "symbol": "MVR",
    "about": "https://en.wikipedia.org/wiki/Maldivian_rufiyaa",
  },
  "MRO": {
    "name": _("Mauritanian Ouguiya (1973–2017)"),
    "flag": "MRO",
    "providers": "1,2",
    "symbol": "MRO",
    "about": "https://en.wikipedia.org/wiki/Mauritanian_ouguiya",
  },
  "MUR": {
    "name": _("Mauritian Rupee"),
    "flag": "🇲🇺",
    "providers": "1,2",
    "symbol": "MUR",
    "about": "https://en.wikipedia.org/wiki/Mauritian_rupee",
  },
  "MXN": {
    "name": _("Mexican Peso"),
    "flag": "🇲🇽",
    "providers": "0,1,2",
    "symbol": "MXN",
    "about": "https://en.wikipedia.org/wiki/Mexican_peso",
  },
  "MDL": {
    "name": _("Moldovan Leu"),
    "flag": "🇲🇩",
    "providers": "1,2",
    "symbol": "MDL",
    "about": "https://en.wikipedia.org/wiki/Moldovan_leu",
  },
  "MAD": {
    "name": _("Moroccan Dirham"),
    "flag": "🇲🇦",
    "providers": "1,2",
    "symbol": "MAD",
    "about": "https://en.wikipedia.org/wiki/Moroccan_dirham",
  },
  "MZM": {
    "name": _("Mozambican metical"),
    "flag": "MZM",
    "providers": "1",
    "symbol": "MZM",
    "about": "https://en.wikipedia.org/wiki/Mozambican_metical",
  },
  "MZN": {
    "name": _("Mozambican metical"),
    "flag": "🇲🇿",
    "providers": "1,2",
    "symbol": "MZN",
    "about": "https://en.wikipedia.org/wiki/Mozambican_metical",
  },
  "MMK": {
    "name": _("Myanmar Kyat"),
    "flag": "🇲🇲",
    "providers": "1,2",
    "symbol": "MMK",
    "about": "https://en.wikipedia.org/wiki/Myanmar_kyat",
  },
  "TWD": {
    "name": _("New Taiwan dollar"),
    "flag": "🇹🇼",
    "providers": "1,2",
    "symbol": "TWD",
    "about": "https://en.wikipedia.org/wiki/New_Taiwan_dollar",
  },
  "NAD": {
    "name": _("Namibian dollar"),
    "flag": "🇳🇦",
    "providers": "1,2",
    "symbol": "NAD",
    "about": "https://en.wikipedia.org/wiki/Namibian_dollar",
  },
  "NPR": {
    "name": _("Nepalese Rupee"),
    "flag": "🇳🇵",
    "providers": "1,2",
    "symbol": "NPR",
    "about": "https://en.wikipedia.org/wiki/Nepalese_rupee",
  },
  "ANG": {
    "name": _("Netherlands Antillean Guilder"),
    "flag": "ANG",
    "providers": "1,2",
    "symbol": "ANG",
    "about": "https://en.wikipedia.org/wiki/Netherlands_Antillean_guilder",
  },
  "NZD": {
    "name": _("New Zealand Dollar"),
    "flag": "🇳🇿",
    "providers": "0,1,2",
    "symbol": "NZD",
    "about": "https://en.wikipedia.org/wiki/New_Zealand_dollar",
  },
  "NIO": {
    "name": _("Nicaraguan Córdoba"),
    "flag": "🇳🇮",
    "providers": "1,2",
    "symbol": "NIO",
    "about": "https://en.wikipedia.org/wiki/Nicaraguan_c%C3%B3rdoba",
  },
  "NGN": {
    "name": _("Nigerian Naira"),
    "flag": "🇳🇬",
    "providers": "1,2",
    "symbol": "NGN",
    "about": "https://en.wikipedia.org/wiki/Nigerian_naira",
  },
  "NOK": {
    "name": _("Norwegian Krone"),
    "flag": "🇳🇴",
    "providers": "0,1,2",
    "symbol": "NOK",
    "about": "https://en.wikipedia.org/wiki/Norwegian_krone",
  },
  "OMR": {
    "name": _("Omani Rial"),
    "flag": "🇴🇲",
    "providers": "1,2",
    "symbol": "OMR",
    "about": "https://en.wikipedia.org/wiki/Omani_rial",
  },
  "PKR": {
    "name": _("Pakistani Rupee"),
    "flag": "🇵🇰",
    "providers": "1,2",
    "symbol": "PKR",
    "about": "https://en.wikipedia.org/wiki/Pakistani_rupee",
  },
  "PAB": {
    "name": _("Panamanian Balboa"),
    "flag": "🇵🇦",
    "providers": "1,2",
    "symbol": "PAB",
    "about": "https://en.wikipedia.org/wiki/Panamanian_balboa",
  },
  "PGK": {
    "name": _("Papua New Guinean Kina"),
    "flag": "🇵🇬",
    "providers": "1,2",
    "symbol": "PGK",
    "about": "https://en.wikipedia.org/wiki/Papua_New_Guinean_kina",
  },
  "PYG": {
    "name": _("Paraguayan Guarani"),
    "flag": "🇵🇾",
    "providers": "1,2",
    "symbol": "PYG",
    "about": "https://en.wikipedia.org/wiki/Guarani_language",
  },
  "PHP": {
    "name": _("Philippine Piso"),
    "flag": "🇵🇭",
    "providers": "0,1,2",
    "symbol": "PHP",
    "about": "https://en.wikipedia.org/wiki/Philippine_peso",
  },
  "PLN": {
    "name": _("Poland złoty"),
    "flag": "🇵🇱",
    "providers": "0,1,2",
    "symbol": "PLN",
    "about": "https://en.wikipedia.org/wiki/Polish_z%C5%82oty",
  },
  "GBP": {
    "name": _("Pound sterling"),
    "flag": "🇬🇧",
    "providers": "0,1,2",
    "symbol": "£",
    "about": "https://en.wikipedia.org/wiki/Pound_sterling",
  },
  "QAR": {
    "name": _("Qatari Rial"),
    "flag": "🇶🇦",
    "providers": "1,2",
    "symbol": "QAR",
    "about": "https://en.wikipedia.org/wiki/Qatari_riyal",
  },
  "ROL": {
    "name": _("Romanian Leu"),
    "flag": "ROL",
    "providers": "1",
    "symbol": "ROL",
    "about": "https://en.wikipedia.org/wiki/Romanian_leu",
  },
  "RON": {
    "name": _("Romanian Leu"),
    "flag": "🇷🇴",
    "providers": "0,1,2",
    "symbol": "RON",
    "about": "https://en.wikipedia.org/wiki/Romanian_leu",
  },
  "RUR": {
    "name": _("Russian Ruble"),
    "flag": "RUR",
    "providers": "1",
    "symbol": "RUR",
    "about": "https://en.wikipedia.org/wiki/Russian_ruble",
  },
  "RUB": {
    "name": _("Russian Ruble"),
    "flag": "🇷🇺",
    "providers": "1,2",
    "symbol": "RUB",
    "about": "https://en.wikipedia.org/wiki/Russian_ruble",
  },
  "RWF": {
    "name": _("Rwandan franc"),
    "flag": "🇷🇼",
    "providers": "1,2",
    "symbol": "RWF",
    "about": "https://en.wikipedia.org/wiki/Rwandan_franc",
  },
  "SVC": {
    "name": _("Salvadoran Colón"),
    "flag": "",
    "providers": "1,2",
    "symbol": "SVC",
    "about": "https://en.wikipedia.org/wiki/Salvadoran_col%C3%B3n",
  },
  "SAR": {
    "name": _("Saudi Riyal"),
    "flag": "🇸🇦",
    "providers": "1,2",
    "symbol": "SAR",
    "about": "https://en.wikipedia.org/wiki/Saudi_riyal",
  },
  "CSD": {
    "name": _("Serbian Dinar"),
    "flag": "CSD",
    "providers": "1",
    "symbol": "CSD",
    "about": "https://en.wikipedia.org/wiki/Serbian_dinar",
  },
  "RSD": {
    "name": _("Serbian Dinar"),
    "flag": "🇷🇸",
    "providers": "1,2",
    "symbol": "RSD",
    "about": "https://en.wikipedia.org/wiki/Serbian_dinar",
  },
  "SCR": {
    "name": _("Seychellois Rupee"),
    "flag": "🇸🇨",
    "providers": "1,2",
    "symbol": "SCR",
    "about": "https://en.wikipedia.org/wiki/Seychellois_rupee",
  },
  "SLL": {
    "name": _("Sierra Leonean Leone"),
    "flag": "🇸🇱",
    "providers": "1,2",
    "symbol": "SLL",
    "about": "https://en.wikipedia.org/wiki/Sierra_Leonean_leone",
  },
  "SGD": {
    "name": _("Singapore Dollar"),
    "flag": "🇸🇬",
    "providers": "0,1,2",
    "symbol": "SGD",
    "about": "https://en.wikipedia.org/wiki/Singapore_dollar",
  },
  "PEN": {
    "name": _("Sol"),
    "flag": "🇵🇪",
    "providers": "1,2",
    "symbol": "PEN",
    "about": "https://en.wikipedia.org/wiki/Peruvian_sol",
  },
  "SBD": {
    "name": _("Solomon Islands Dollar"),
    "flag": "🇸🇧",
    "providers": "1,2",
    "symbol": "SBD",
    "about": "https://en.wikipedia.org/wiki/Solomon_Islands_dollar",
  },
  "SOS": {
    "name": _("Somali Shilling"),
    "flag": "🇸🇴",
    "providers": "1,2",
    "symbol": "SOS",
    "about": "https://en.wikipedia.org/wiki/Somali_shilling",
  },
  "ZAR": {
    "name": _("South African Rand"),
    "flag": "🇿🇦",
    "providers": "0,1,2",
    "symbol": "ZAR",
    "about": "https://en.wikipedia.org/wiki/South_African_rand",
  },
  "KRW": {
    "name": _("South Korean won"),
    "flag": "🇰🇷",
    "providers": "0,1,2",
    "symbol": "₩",
    "about": "https://en.wikipedia.org/wiki/South_Korean_won",
  },
  "VEF": {
    "name": _("Sovereign Bolivar"),
    "flag": "🇻🇪",
    "providers": "1",
    "symbol": "VEF",
    "about": "https://en.wikipedia.org/wiki/Venezuelan_bol%C3%ADvar",
  },
  "XDR": {
    "name": _("Special Drawing Rights"),
    "flag": "XDR",
    "providers": "1,2",
    "symbol": "XDR",
    "about": "https://en.wikipedia.org/wiki/Special_drawing_rights",
  },
  "LKR": {
    "name": _("Sri Lankan Rupee"),
    "flag": "🇱🇰",
    "providers": "1,2",
    "symbol": "LKR",
    "about": "https://en.wikipedia.org/wiki/Sri_Lankan_rupee",
  },
  "SSP": {
    "name": _("Sudanese pound"),
    "flag": "🇸🇸",
    "providers": "1,2",
    "symbol": "SSP",
    "about": "https://en.wikipedia.org/wiki/Sudanese_pound",
  },
  "SDG": {
    "name": _("Sudanese pound"),
    "flag": "🇸🇩",
    "providers": "1,2",
    "symbol": "SDG",
    "about": "https://en.wikipedia.org/wiki/Sudanese_pound",
  },
  "SRD": {
    "name": _("Surinamese Dollar"),
    "flag": "🇸🇷",
    "providers": "1,2",
    "symbol": "SRD",
    "about": "https://en.wikipedia.org/wiki/Surinamese_dollar",
  },
  "SZL": {
    "name": _("Swazi Lilangeni"),
    "flag": "🇸🇿",
    "providers": "1,2",
    "symbol": "SZL",
    "about": "https://en.wikipedia.org/wiki/Swazi_lilangeni",
  },
  "SEK": {
    "name": _("Swedish Krona"),
    "flag": "🇸🇪",
    "providers": "0,1,2",
    "symbol": "SEK",
    "about": "https://en.wikipedia.org/wiki/Swedish_krona",
  },
  "CHF": {
    "name": _("Swiss Franc"),
    "flag": "🇨🇭",
    "providers": "0,1,2",
    "symbol": "CHF",
    "about": "https://en.wikipedia.org/wiki/Swiss_franc",
  },
  "TJS": {
    "name": _("Tajikistani Somoni"),
    "flag": "🇹🇯",
    "providers": "1,2",
    "symbol": "TJS",
    "about": "https://en.wikipedia.org/wiki/Tajikistani_somoni",
  },
  "TZS": {
    "name": _("Tanzanian Shilling"),
    "flag": "🇹🇿",
    "providers": "1,2",
    "symbol": "TZS",
    "about": "https://en.wikipedia.org/wiki/Tanzanian_shilling",
  },
  "THB": {
    "name": _("Thai Baht"),
    "flag": "🇹🇭",
    "providers": "0,1,2",
    "symbol": "THB",
    "about": "https://en.wikipedia.org/wiki/Thai_baht",
  },
  "TOP": {
    "name": _("Tongan Paʻanga"),
    "flag": "🇹🇴",
    "providers": "1,2",
    "symbol": "TOP",
    "about": "https://en.wikipedia.org/wiki/Tongan_pa%CA%BBanga",
  },
  "TTD": {
    "name": _("Trinidad & Tobago Dollar"),
    "flag": "🇹🇹",
    "providers": "1,2",
    "symbol": "TTD",
    "about": "https://en.wikipedia.org/wiki/Trinidad_and_Tobago_dollar",
  },
  "TND": {
    "name": _("Tunisian Dinar"),
    "flag": "🇹🇳",
    "providers": "1,2",
    "symbol": "TND",
    "about": "https://en.wikipedia.org/wiki/Tunisian_dinar",
  },
  "TRY": {
    "name": _("Turkish lira"),
    "flag": "🇹🇷",
    "providers": "0,1,2",
    "symbol": "TRY",
    "about": "https://en.wikipedia.org/wiki/Turkish_lira",
  },
  "TMM": {
    "name": _("Turkmenistan manat"),
    "flag": "TMT",
    "providers": "1",
    "symbol": "TMM",
    "about": "https://en.wikipedia.org/wiki/Turkmenistani_manat",
  },
  "TMT": {
    "name": _("Turkmenistan manat"),
    "flag": "🇹🇲",
    "providers": "1,2",
    "symbol": "TMT",
    "about": "https://en.wikipedia.org/wiki/Turkmenistani_manat",
  },
  "UGX": {
    "name": _("Ugandan Shilling"),
    "flag": "🇺🇬",
    "providers": "1,2",
    "symbol": "UGX",
    "about": "https://en.wikipedia.org/wiki/Ugandan_shilling",
  },
  "UAH": {
    "name": _("Ukrainian hryvnia"),
    "flag": "🇺🇦",
    "providers": "1,2",
    "symbol": "UAH",
    "about": "https://en.wikipedia.org/wiki/Ukrainian_hryvnia",
  },
  "AED": {
    "name": _("United Arab Emirates Dirham"),
    "flag": "🇦🇪",
    "providers": "1,2",
    "symbol": "AED",
    "about": "https://en.wikipedia.org/wiki/United_Arab_Emirates_dirham",
  },
  "USD": {
    "name": _("United States Dollar"),
    "flag": "🇺🇸",
    "providers": "0,1,2",
    "symbol": "$",
    "about": "https://en.wikipedia.org/wiki/United_States_dollar",
  },
  "UYU": {
    "name": _("Uruguayan Peso"),
    "flag": "🇺🇾",
    "providers": "1,2",
    "symbol": "UYU",
    "about": "https://en.wikipedia.org/wiki/Uruguayan_peso",
  },
  "UZS": {
    "name": _("Uzbekistani Som"),
    "flag": "🇺🇿",
    "providers": "1,2",
    "symbol": "UZS",
    "about": "https://en.wikipedia.org/wiki/Uzbekistani_sum",
  },
  "VND": {
    "name": _("Vietnamese dong"),
    "flag": "🇻🇳",
    "providers": "1,2",
    "symbol": "₫",
    "about": "https://en.wikipedia.org/wiki/Vietnamese_%C4%91%E1%BB%93ng",
  },
  "XOF": {
    "name": _("West African CFA franc"),
    "flag": "",
    "providers": "1,2",
    "symbol": "XOF",
    "about": "https://en.wikipedia.org/wiki/West_African_CFA_franc",
  },
  "YER": {
    "name": _("Yemeni Rial"),
    "flag": "🇾🇪",
    "providers": "1,2",
    "symbol": "YER",
    "about": "https://en.wikipedia.org/wiki/Yemeni_rial",
  },
  "ZMW": {
    "name": _("Zambian Kwacha"),
    "flag": "🇿🇲",
    "providers": "1,2",
    "symbol": "ZMW",
    "about": "https://en.wikipedia.org/wiki/Zambian_kwacha",
  },
}



# convertion.py
#
# Copyright 2023 Ideve Core
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import gi
from typing import Any, Dict, Union

gi.require_version("Adw", "1")
gi.require_version("Gtk", "4.0")

from gi.repository import Adw, Gio, Gtk
from ...components import CurrencySelector
from ...utils import CurrenciesListModel
from ...define import RES_PATH, CODES

resource = f"{RES_PATH}/pages/convertion/index.ui"

def convertion_page(application: Adw.Application, from_currency_value):
    builder = Gtk.Builder.new_from_resource(resource)
    settings = application.utils.settings
    convertion = application.utils.convertion
    page = builder.get_object("toast_overlay")
    from_currency_selector: CurrencySelector = builder.get_object("from_currency_selector")
    to_currency_selector: CurrencySelector = builder.get_object("to_currency_selector")
    from_currency_entry = builder.get_object("from_currency_entry")
    to_currency_entry = builder.get_object("to_currency_entry")
    stack = builder.get_object("stack")
    reload = builder.get_object("reload")
    toast_overlay = builder.get_object("toast_overlay")
    to_currency_value = 0
    def load_currencies(provider: int):
        codes = {currency: details for currency, details in CODES.items() if str(provider) in details['providers']}
        from_currency_model = CurrenciesListModel(currency_names_func)
        to_currency_model = CurrenciesListModel(currency_names_func)
        from_currency_selector.bind_models(from_currency_model)
        from_currency_model.set_currencies(codes)
        to_currency_selector.bind_models(to_currency_model)
        to_currency_model.set_currencies(codes)
        if not settings.get_string('src-currency') in codes or not settings.get_string('dest-currency') in codes:
            settings.set_string("src-currency", "USD")
            settings.set_string("dest-currency", "EUR")

        from_currency_selector.set_selected(settings.get_string('src-currency'))
        to_currency_selector.set_selected(settings.get_string('dest-currency'))

    def change_provider(settings, key):
        stack.set_visible_child_name("loading")
        load_currencies(settings.get_enum(key))
        convert(from_currency_entry.get_text(), force=True)

    def is_loading():
        return stack.get_visible_child_name() == "loading"

    def currency_names_func(code):
        name = gettext(CODES.get(code, '')['name'])
        return name if name else None

    def valid_from_currency_value(value: str):
        if not value:
            return False
        try:
            f=float(value)
            return True
        except ValueError:
            return False

    def convert(value, force=False):
        if not is_loading() and value or force:
            value = application.utils.parse_number(value)
            if not value:
                from_currency_entry.add_css_class("error")
                return
            from_currency_entry.remove_css_class("error")
            def thread_cb(task: Gio.Task, self, task_data: object, cancellable: Gio.Cancellable):
                try:
                    convertion.convert(float(value), from_currency_selector.selected, to_currency_selector.selected, settings.get_enum("providers"))
                    task.return_value(self.__converted_data)
                except Exception as e:
                    task.return_value(e)
            if convertion.converted_data["from"] != from_currency_selector.selected or convertion.converted_data["to"] != to_currency_selector.selected or not convertion.converted_data["converted"] or force:
                stack.set_visible_child_name("loading")
                task = Gio.Task.new(application, None, None, None)
                task.run_in_thread(thread_cb)
            else:
                convertion.convert(float(value), from_currency_selector.selected, to_currency_selector.selected, settings.get_enum("providers"))

    def converted(data: Dict[str, Union[str, int]]):
        if not data["converted"]:
            stack.set_visible_child_name("convertion-error")
            toast_overlay.add_toast(Adw.Toast.new(
                title = _("Error converting, please try again."),
            ))
        else:
            stack.set_visible_child_name("result")
            result = application.utils.format_number(str(data["amount"]))
            if result:
                to_currency_entry.set_text(result)
            else:
                stack.set_visible_child_name("convertion-error")

    def currency_selectors_changed(_obj, _param):
        from_code = from_currency_selector.selected
        to_code = to_currency_selector.selected
        if from_code != to_code:
          if settings.get_string("src-currency") != from_code:
            settings.set_string('src-currency', from_code)
          if settings.get_string("dest-currency") != to_code:
            settings.set_string('dest-currency', to_code)
          convert(from_currency_entry.get_text())

    load_currencies(settings.get_enum("providers"))
    from_currency_entry.connect('changed', lambda entry: convert(entry.get_text()))
    from_currency_selector.connect('notify::selected', currency_selectors_changed)
    to_currency_selector.connect('notify::selected', currency_selectors_changed)
    reload.connect('clicked', lambda button: convert(from_currency_entry.get_text()))
    convertion.connect("converted", converted)
    settings.connect("changed::providers", change_provider)
    settings.connect("changed::src-currency", lambda settings, key: from_currency_selector.set_selected(settings.get_string(key)))
    settings.connect("changed::dest-currency", lambda settings, key: to_currency_selector.set_selected(settings.get_string(key)))
    settings.connect("changed::high-precision", lambda settings, key: convert(from_currency_entry.get_text()))

    if from_currency_value:
        from_currency_entry.set_text(str(from_currency_value))
    else:
        from_currency_entry.set_text("1")

    return page



# window.py
#
# Copyright 2023 Ideve Core
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import gi, math

gi.require_version("Adw", "1")
gi.require_version("Gtk", "4.0")

from typing import Union, Any, Dict
from gi.repository import Adw, Gdk, Gio, GLib, Gtk
from .define import RES_PATH
from .pages import convertion_page
from .components import Shortcuts

resource = f"{RES_PATH}/window.ui"

def string_to_color(string: str):
    colors = {
        "light": Adw.ColorScheme.FORCE_LIGHT,
        "default": Adw.ColorScheme.DEFAULT,
        "dark": Adw.ColorScheme.FORCE_DARK,
    }
    return colors[string]

def create_main_window(application: Adw.Application, from_currency_value: int):
    builder = Gtk.Builder.new_from_resource(resource)
    settings = application.utils.settings
    convertion = application.utils.convertion
    window = builder.get_object("window")
    content = builder.get_object("content")
    menu_button = builder.get_object("menu_button")
    info = builder.get_object("info")
    source = builder.get_object("source")
    providers_action_group = Gio.SimpleActionGroup.new();
    window.insert_action_group("window", providers_action_group);
    providers_action_group.add_action(settings.create_action('providers'));

    def load_window_state():
        settings.bind(
            key="width",
            object=window,
            property="default-width",
            flags=Gio.SettingsBindFlags.DEFAULT,
        )
        settings.bind(
            key="height",
            object=window,
            property="default-height",
            flags=Gio.SettingsBindFlags.DEFAULT,
        )
        settings.bind(
            key="is-maximized",
            object=window,
            property="maximized",
            flags=Gio.SettingsBindFlags.DEFAULT,
        )
        settings.bind(
            key="is-fullscreen",
            object=window,
            property="fullscreened",
            flags=Gio.SettingsBindFlags.DEFAULT,
        )
        window.set_help_overlay(Shortcuts())

    def converted(data: Dict[str, Union[str, int]]):
        info.set_text(f'{data["info"]} -')
        source.set_label(application.utils.settings.get_string("providers").upper())
        source.set_uri(convertion.get_convertion()['disclaimer'])
        source.set_visible(True)

    def load_convertion_page(from_currency_value: int = 0):
        content.set_child(convertion_page(application, from_currency_value))

    def open_uri(link: str):
        Gtk.show_uri(
          window,
          link,
          Gdk.CURRENT_TIME
        );

    load_window_state()
    convertion.connect("converted", converted)
    load_convertion_page(from_currency_value)
    window.set_application(application)
    window.set_icon_name(application.get_application_id())
    window.load_convertion_page = load_convertion_page
    return window



# currency_selector_row.py
#
# Copyright 2023 Ideve Core
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import textwrap
from gi.repository import GObject, Gtk
from ...define import CODES

@Gtk.Template(resource_path='/io/github/idevecore/Valuta/components/currency_selector_row/index.ui')
class CurrencySelectorRow(Gtk.ListBoxRow):
    __gtype_name__ = 'CurrencySelectorRow'

    name = Gtk.Template.Child()
    selection = Gtk.Template.Child()
    
    def __init__(self, currency):
        super().__init__()
        self.currency = currency
        #self.name.props.label = f'{CODES[str(self.currency)]["flag"]}   {self.currency} – {self.currency.name}'
        self.name.props.label = f'{self.currency} – {textwrap.shorten(self.currency.name, width=30, placeholder="...")}'

        self.currency.bind_property(
            'selected',
            self.selection,
            'visible',
            GObject.BindingFlags.SYNC_CREATE
        )



# requests.py
#
# Copyright 2023 Ideve Core
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any, Dict, Union
from datetime import datetime
import gi, json, re
gi.require_version('Soup', '3.0')
from gi.repository import Soup, GLib
from .define import BASE_URL_LANG_PREFIX, CODES

class Providers:
    ECB_BASE_URL: str = 'https://api.frankfurter.app/latest'
    response = {
        "from": "",
        "to": "",
        "amount": "",
        "converted": False,
        "info": "",
        "disclaimer": "",
        "provider": "",
    }

    def __init__(self, from_currency: str, to_currency: str, from_currency_value: int):
        self.from_currency = from_currency
        self.to_currency = to_currency
        self.from_currency_value = from_currency_value

    def mount_url(self):
        pass

    def serializer(self):
        pass

    def default_response(self):
        pass

    def create_info(self, date: str, time: str = "00:00:00"):
        date = date.split("-")
        time = time.split(":")
        date_time = GLib.DateTime.new_local(float(date[0]), float(date[1]), float(date[2]), float(time[0]), float(time[1]), float(time[2]))
        return date_time.format("%B %e, %Y")

class ECB(Providers):
    def mount_url(self):
        return f'{self.ECB_BASE_URL}?amount={self.from_currency_value}&from={self.from_currency}&to={self.to_currency}'

    def serializer(self, data: bytes) -> Dict[str, Union[str, int]]:
        return self.default_response(json.loads(data))

    def default_response(self, data: Dict[str, str]):
        self.response["base"] = data["rates"][self.to_currency]
        self.response["from"] = self.from_currency
        self.response["to"] = self.to_currency
        self.response["amount"] = 0
        self.response["info"] = self.create_info(data["date"])
        self.response["disclaimer"] = self.mount_url()
        self.response["provider"] = 0
        return self.response

providers = {
    0 : ECB,
}

class SoupSession(Soup.Session):
    def __init__(self):
        Soup.Session.__init__(self)

    def create_request(self, method: str, url: str, headers: dict = {}) -> Soup.Message:
        """ Helper for creating Soup.Message """
        message = Soup.Message.new(method, url)
        if headers:
            for name, value in headers.items():
                message.get_request_headers().append(name, value)
        if 'User-Agent' not in headers:
            message.get_request_headers().append('User-Agent', 'Currency Converter')
        return message

    def get_response(self, message: Soup.Message):
        response = None
        try:
            response = self.send_and_read(message, None)
            data = response.get_data()
            return data
        except GLib.GError as error:
             raise error from error

class Requests:
    HEADERS: Dict[str, str] = {
        'User-agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/117.0',
    }
    def __init__(self, provider: int, from_currency: str, to_currency: str, from_currency_value: int):
        self.__provider = providers[provider](from_currency, to_currency, from_currency_value)
        self.__url = self.__provider.mount_url()
    def get(self):
        session = SoupSession();
        message = session.create_request("GET", self.__url, self.HEADERS)
        try:
            return self.__provider.serializer(session.get_response(message))
        except Exception as error:
            return error.message



# about.py
#
# Copyright 2023 Ideve Core
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import gi

gi.require_version('Adw', '1')

from gi.repository import Adw, Gio, GLib, Gtk, GObject
from .define import APP_ID, VERSION

def about(application: Adw.Application):
    def get_flatpak_info():
        key_file = GLib.KeyFile.new();
        try:
            key_file.load_from_file("/.flatpak-info", GLib.KeyFileFlags.NONE);
        except Exception as error:
            return None;
        return key_file.get_value("Instance", "flatpak-version");

    app_info = f'{APP_ID} {VERSION}'
    glib_os_info = f'{GLib.get_os_info("ID")} {GLib.get_os_info("VERSION_ID")}'
    pygobject_info = f'PyGObject {".".join(map(str, GObject.pygobject_version))}'
    adw_info = f'Adw {Adw.MAJOR_VERSION}'
    gtk_info = f'Gtk {Gtk.get_major_version()}'
    glib_info = f'GLib {".".join(map(str, GLib.glib_version))}'
    flatpak_info = f'Flatpak {get_flatpak_info()}'
    blueprint_info = 'Blueprint 0.10.0'

    debug_info = f'{app_info}\n{glib_os_info}\n{pygobject_info}\n{adw_info}\n{gtk_info}\n{glib_info}\n{flatpak_info}\n{blueprint_info}'

    """Callback for the app.about action."""

    return Adw.AboutDialog(
        application_name=gettext('Valuta'),
        application_icon=APP_ID,
        developer_name='Ideve Core',
        version=VERSION,
        developers=['Ideve Core'],
        designers=['Brage Fuglseth https://bragefuglseth.dev'],
        issue_url='https://github.com/ideveCore/valuta/issues',
        debug_info=debug_info,
        license_type=Gtk.License.GPL_3_0,
        copyright='© 2023 Ideve Core'
    )
